@@ -16,7 +16,6 @@ import argparse
 import glob
 import itertools
 import logging
-import netrc
 import operator
 import os
 import shutil
@@ -28,6 +27,8 @@ from uuid import uuid4
 
 import daiquiri
 
+from git_pull_request import pagure
+
 import github
 
 
@@ -48,29 +49,43 @@ def _run_shell_command(cmd, output=None, raise_on_error=True):
         return out[0].strip().decode()
 
 
-def get_login_password(site_name="github.com",
-                       netrc_file="~/.netrc",
-                       git_credential_file="~/.git-credentials"):
-    """Read a .netrc file and return login/password for LWN."""
-    try:
-        n = netrc.netrc(os.path.expanduser(netrc_file))
-    except OSError:
-        pass
-    else:
-        if site_name in n.hosts:
-            return n.hosts[site_name][0], n.hosts[site_name][2]
+def get_login_password(protocol="https", host="github.com"):
+    """Get login/password from git credential."""
+    subp = subprocess.Popen(["git", "credential", "fill"],
+                            stdin=subprocess.PIPE,
+                            stdout=subprocess.PIPE)
+    # TODO add path support
+    request = "protocol={}\nhost={}\n".format(protocol, host).encode()
+    username = None
+    password = None
+    stdout, stderr = subp.communicate(input=request)
+    ret = subp.wait()
+    if ret != 0:
+        LOG.error("git credential returned exited with status %d", ret)
+        return None, None
+    for line in stdout.split(b'\n'):
+        key, _, value = line.partition(b"=")
+        if key == b"username":
+            username = value.decode()
+        elif key == b"password":
+            password = value.decode()
+        if username and password:
+            break
+    return username, password
 
-    try:
-        with open(os.path.expanduser(git_credential_file)) as f:
-            for line in f:
-                parsed = parse.urlparse(line.strip())
-                if parsed.hostname == site_name:
-                    return (parse.unquote(parsed.username),
-                            parse.unquote(parsed.password))
-    except OSError:
-        pass
 
-    return None, None
+def approve_login_password(user, password,
+                           host="github.com", protocol="https"):
+    """Tell git to approve the credential."""
+    subp = subprocess.Popen(["git", "credential", "approve"],
+                            stdin=subprocess.PIPE,
+                            stdout=subprocess.PIPE)
+    request = "protocol={}\nhost={}\nusername={}\npassword={}\n".format(
+        protocol, host, user, password).encode()
+    subp.communicate(input=request)
+    ret = subp.wait()
+    if ret != 0:
+        LOG.error("git credential returned exited with status %d", ret)
 
 
 def git_remote_matching_url(url):
@@ -111,11 +126,33 @@ def git_get_remote_branch_for_branch(branch):
     return branch
 
 
-def get_github_hostname_user_repo_from_url(url):
-    """Return hostname, user and repository to fork from.
+def git_get_config_hosttype():
+    return _run_shell_command(
+        ["git", "config", "git-pull-request.hosttype"],
+        output=True, raise_on_error=False)
+
+
+def git_set_config_hosttype(hosttype):
+    _run_shell_command(
+        ["git", "config", "git-pull-request.hosttype", hosttype])
+
+
+def get_hosttype(host):
+    hosttype = git_get_config_hosttype()
+    if hosttype == "":
+        if pagure.is_pagure(host):
+            hosttype = "pagure"
+        else:
+            hosttype = "github"
+        git_set_config_hosttype(hosttype)
+    return hosttype
+
+
+def get_hosttype_hostname_user_repo_from_url(url):
+    """Return hostype, hostname, user and repository to fork from.
 
     :param url: The URL to parse
-    :return: hostname, user, repository
+    :return: hosttype, hostname, user, repository
     """
     parsed = parse.urlparse(url)
     if parsed.netloc == '':
@@ -126,8 +163,12 @@ def get_github_hostname_user_repo_from_url(url):
     else:
         path = parsed.path[1:].rstrip('/')
         host = parsed.netloc
-    user, repo = path.split("/", 1)
-    return host, user, repo[:-4] if repo.endswith('.git') else repo
+    hosttype = get_hosttype(host)
+    if hosttype == "pagure":
+        user, repo = None, path
+    else:
+        user, repo = path.split("/", 1)
+    return hosttype, host, user, repo[:-4] if repo.endswith('.git') else repo
 
 
 def split_and_remove_empty_lines(s):
@@ -183,7 +224,6 @@ def git_get_title_and_message(begin, end):
 def git_pull_request(target_remote=None, target_branch=None,
                      title=None, message=None,
                      comment=None,
-                     comment_on_update=True,
                      rebase=True,
                      force_editor=False,
                      download=None,
@@ -224,41 +264,43 @@ def git_pull_request(target_remote=None, target_branch=None,
 
     LOG.debug("Remote URL for remote `%s' is `%s'", target_remote, target_url)
 
-    hostname, user_to_fork, reponame_to_fork = (
-        get_github_hostname_user_repo_from_url(target_url)
+    hosttype, hostname, user_to_fork, reponame_to_fork = (
+        get_hosttype_hostname_user_repo_from_url(target_url)
     )
-    LOG.debug("GitHub user and repository to fork: %s/%s on %s",
-              user_to_fork, reponame_to_fork, hostname)
+    LOG.debug("%s user and repository to fork: %s/%s on %s",
+              hosttype.capitalize(), user_to_fork, reponame_to_fork, hostname)
 
-    user, password = get_login_password(hostname)
-    if not user or not password:
+    user, password = get_login_password(host=hostname)
+    if not user and not password:
         LOG.critical(
-            "Unable to find your GitHub credentials for %s.\n"
-            "Make sure you have a line like this in your ~/.netrc file:\n"
-            "machine %s login <login> password <pwd>, "
-            "or use git store credentials",
-            hostname, hostname
+            "Unable to find your credentials for %s.\n"
+            "Make sure you have a git credential working.",
+            hostname,
         )
         return 35
 
-    LOG.debug("Found GitHub user: `%s' password: <redacted>", user)
+    LOG.debug("Found %s user: `%s' password: <redacted>", hostname, user)
 
-    kwargs = {}
-    if hostname != "github.com":
-        kwargs['base_url'] = "https://" + hostname + "/api/v3"
-        LOG.debug("Using API base url `%s'", kwargs['base_url'])
-
-    g = github.Github(user, password, **kwargs)
-    repo = g.get_user(user_to_fork).get_repo(reponame_to_fork)
+    if hosttype == "pagure":
+        g = pagure.Client(hostname, user, password, reponame_to_fork)
+        repo = g.get_repo(reponame_to_fork)
+    else:
+        kwargs = {}
+        if hostname != "github.com":
+            kwargs['base_url'] = "https://" + hostname + "/api/v3"
+            LOG.debug("Using API base url `%s'", kwargs['base_url'])
+        g = github.Github(user, password, **kwargs)
+        repo = g.get_user(user_to_fork).get_repo(reponame_to_fork)
 
     if download is not None:
         download_pull_request(g, repo, target_remote, download)
     else:
-        fork_and_push_pull_request(g, repo, rebase, target_remote,
+        fork_and_push_pull_request(g, hosttype, repo, rebase, target_remote,
                                    target_branch, branch, user, title, message,
-                                   comment_on_update, comment,
+                                   comment,
                                    force_editor, tag_previous_revision,
                                    dont_fork)
+    approve_login_password(host=hostname, user=user, password=password)
 
 
 def download_pull_request(g, repo, target_remote, pull_number):
@@ -340,9 +382,9 @@ def preserve_older_revision(branch, remote_to_push):
     _run_shell_command(["git", "tag", "-d", tag])
 
 
-def fork_and_push_pull_request(g, repo_to_fork, rebase, target_remote,
-                               target_branch, branch, user, title, message,
-                               comment_on_update, comment,
+def fork_and_push_pull_request(g, hosttype, repo_to_fork, rebase,
+                               target_remote, target_branch, branch, user,
+                               title, message, comment,
                                force_editor, tag_previous_revision, dont_fork):
 
     g_user = g.get_user()
@@ -360,7 +402,7 @@ def fork_and_push_pull_request(g, repo_to_fork, rebase, target_remote,
             LOG.debug("Found forked repository already in remote as `%s'",
                       remote_to_push)
         else:
-            remote_to_push = "github"
+            remote_to_push = hosttype
             _run_shell_command(
                 ["git", "remote", "add",
                  remote_to_push, repo_forked.clone_url])
@@ -415,11 +457,7 @@ def fork_and_push_pull_request(g, repo_to_fork, rebase, target_remote,
             elif message:
                 pull.edit(body=message)
                 LOG.debug("Updated pull-request message")
-        if comment_on_update:
-            if not comment:
-                branch_sha = _run_shell_command(
-                    ["git", "rev-parse", branch], output=True)
-                comment = "Pull-request updated, HEAD is now %s" % branch_sha
+        if comment:
             # FIXME(jd) we should be able to comment directly on a PR without
             # getting it as an issue but pygithub does not allow that yet
             repo_to_fork.get_issue(pull.number).create_comment(comment)
@@ -500,12 +538,6 @@ def main():
         action="store_true",
         default=False,
         help="Force editor to run to edit pull-request message.")
-    parser.add_argument(
-        "--no-comment-on-update",
-        action="store_true",
-        default=False,
-        help="Do not post a comment stating the pull-request has been updated."
-    )
     parser.add_argument(
         "--comment", "-C",
         help="Comment to publish when updating the pull-request"
@@ -540,7 +572,6 @@ def main():
             target_branch=args.target_branch,
             title=args.title,
             message=args.message,
-            comment_on_update=not args.no_comment_on_update,
             comment=args.comment,
             rebase=not args.no_rebase,
             force_editor=args.force_editor,
@@ -550,6 +581,7 @@ def main():
         )
     except Exception:
         LOG.error("Unable to send pull request", exc_info=True)
+        return 128
 
 
 if __name__ == '__main__':
